@@ -26,7 +26,7 @@
 #       You may only use this script to uninstall the version
 #       shown below. Simply run this script to remove Shorewall Firewall
 
-VERSION=4.5.16.1
+VERSION=4.5.21.1
 
 usage() # $1 = exit status
 {
@@ -118,14 +118,14 @@ fi
 
 if [ -L ${SHAREDIR}/shorewall-lite/init ]; then
     FIREWALL=$(readlink -m -q ${SHAREDIR}/shorewall-lite/init)
-elIF [ -n "$INITFILE" ]; then
+elif [ -n "$INITFILE" ]; then
     FIREWALL=${INITDIR}/${INITFILE}
 fi
 
 if [ -f "$FIREWALL" ]; then
     if mywhich updaterc.d ; then
 	updaterc.d shorewall-lite remove
-    elif if mywhich insserv ; then
+    elif mywhich insserv ; then
         insserv -r $FIREWALL
     elif [ mywhich chkconfig ; then
 	chkconfig --del $(basename $FIREWALL)
